﻿using Plugin.Maui.Audio;

namespace TestProject;

/// <summary>
/// 메인 페이지
/// </summary>
public partial class MainPage : ContentPage
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 오디오 관리자
    /// </summary>
    private IAudioManager audioManager;

    /// <summary>
    /// 오디오 재생자
    /// </summary>
    private IAudioPlayer audioPlayer;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - MainPage()

    /// <summary>
    /// 생성자
    /// </summary>
    public MainPage()
    {
        InitializeComponent();

        this.audioManager = new AudioManager();

        Loaded                                += ContentPage_Loaded;
        this.playTapGestureRecognizer.Tapped  += playTapGestureRecognizer_Tapped;
        this.volumeSlider.ValueChanged        += volumeSlider_ValueChanged;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region 컨텐트 페이지 로드시 처리하기 - ContentPage_Loaded(sender, e)

    /// <summary>
    /// 컨텐트 페이지 로드시 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private async void ContentPage_Loaded(object sender, EventArgs e)
    {
        this.audioPlayer = this.audioManager.CreatePlayer(await FileSystem.OpenAppPackageFileAsync("sample.mp3"));

        this.audioPlayer.Volume = this.volumeSlider.Value;

        this.audioPlayer.PlaybackEnded += audioPlayer_PlaybackEnded;
    }

    #endregion
    #region 오디오 재생자 재생 종료시 처리하기 - audioPlayer_PlaybackEnded(sender, e)

    /// <summary>
    /// 오디오 재생자 재생 종료시 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void audioPlayer_PlaybackEnded(object sender, EventArgs e)
    {
        this.playTapGestureRecognizer.Tapped  -= playTapGestureRecognizer_Tapped;
        this.pauseTapGestureRecognizer.Tapped -= pauseTapGestureRecognizer_Tapped;
        this.stopTapGestureRecognizer.Tapped  -= stopTapGestureRecognizer_Tapped;

        this.playTapGestureRecognizer.Tapped  += playTapGestureRecognizer_Tapped;

        this.playFontImageSource.Color  = Colors.Black;
        this.pauseFontImageSource.Color = Colors.Gray;
        this.stopFontImageSource.Color  = Colors.Gray;
    }

    #endregion
    #region 재생 탭 제스처 인식기 탭 처리하기 - playTapGestureRecognizer_Tapped(sender, e)

    /// <summary>
    /// 재생 탭 제스처 인식기 탭 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void playTapGestureRecognizer_Tapped(object sender, EventArgs e)
    {
        this.playTapGestureRecognizer.Tapped  -= playTapGestureRecognizer_Tapped;
        this.pauseTapGestureRecognizer.Tapped -= pauseTapGestureRecognizer_Tapped;
        this.stopTapGestureRecognizer.Tapped  -= stopTapGestureRecognizer_Tapped;

        this.pauseTapGestureRecognizer.Tapped += pauseTapGestureRecognizer_Tapped;
        this.stopTapGestureRecognizer.Tapped  += stopTapGestureRecognizer_Tapped;

        this.playFontImageSource.Color  = Colors.Gray;
        this.pauseFontImageSource.Color = Colors.Black;
        this.stopFontImageSource.Color  = Colors.Black;

        this.audioPlayer.Play();
    }

    #endregion
    #region 중지 탭 제스처 인식기 탭 처리하기 - pauseTapGestureRecognizer_Tapped(sender, e)

    /// <summary>
    /// 중지 탭 제스처 인식기 탭 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void pauseTapGestureRecognizer_Tapped(object sender, EventArgs e)
    {
        this.playTapGestureRecognizer.Tapped  -= playTapGestureRecognizer_Tapped;
        this.pauseTapGestureRecognizer.Tapped -= pauseTapGestureRecognizer_Tapped;
        this.stopTapGestureRecognizer.Tapped  -= stopTapGestureRecognizer_Tapped;

        this.playTapGestureRecognizer.Tapped  += playTapGestureRecognizer_Tapped;
        this.stopTapGestureRecognizer.Tapped  += stopTapGestureRecognizer_Tapped;

        this.playFontImageSource.Color  = Colors.Black;
        this.pauseFontImageSource.Color = Colors.Gray;
        this.stopFontImageSource.Color  = Colors.Black;

        this.audioPlayer.Pause();
    }

    #endregion
    #region 중단 탭 제스처 인식기 탭 처리하기 - stopTapGestureRecognizer_Tapped(sender, e)

    /// <summary>
    /// 중단 탭 제스처 인식기 탭 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void stopTapGestureRecognizer_Tapped(object sender, EventArgs e)
    {
        this.playTapGestureRecognizer.Tapped  -= playTapGestureRecognizer_Tapped;
        this.pauseTapGestureRecognizer.Tapped -= pauseTapGestureRecognizer_Tapped;
        this.stopTapGestureRecognizer.Tapped  -= stopTapGestureRecognizer_Tapped;

        this.playTapGestureRecognizer.Tapped  += playTapGestureRecognizer_Tapped;

        this.playFontImageSource.Color  = Colors.Black;
        this.pauseFontImageSource.Color = Colors.Gray;
        this.stopFontImageSource.Color  = Colors.Gray;

        this.audioPlayer.Stop();
    }

    #endregion
    #region 볼륨 슬라이더 값 변경시 처리하기 - volumeSlider_ValueChanged(sender, e)

    /// <summary>
    /// 볼륨 슬라이더 값 변경시 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void volumeSlider_ValueChanged(object sender, ValueChangedEventArgs e)
    {
        this.audioPlayer.Volume = this.volumeSlider.Value;
    }

    #endregion
}